@@ -7,19 +7,116 @@ Usage:
   >>> for i in trange(10): #same as: for i in tqdm(xrange(10))
   ...     ...
 """
-# future division is important to divide integers and get as
-# a result precise floating numbers (instead of truncated int)
-from __future__ import division, absolute_import
-# import compatibility functions and utilities
+from __future__ import absolute_import
+# integer division / : float, // : int
+from __future__ import division
+# compatibility functions and utilities
 from ._utils import _supports_unicode, _environ_cols_wrapper, _range, _unich, \
-    _term_move_up, _unicode, WeakSet
+    _term_move_up, _unicode, WeakSet, _basestring, _OrderedDict
+# native libraries
 import sys
+from numbers import Number
+from threading import Thread
 from time import time
+from time import sleep
 
 
 __author__ = {"github.com/": ["noamraph", "obiwanus", "kmike", "hadim",
                               "casperdcl", "lrq3000"]}
-__all__ = ['tqdm', 'trange']
+__all__ = ['tqdm', 'trange',
+           'TqdmTypeError', 'TqdmKeyError', 'TqdmDeprecationWarning']
+
+
+class TqdmTypeError(TypeError):
+    pass
+
+
+class TqdmKeyError(KeyError):
+    pass
+
+
+class TqdmDeprecationWarning(Exception):
+    # not suppressed if raised
+    def __init__(self, msg, fp_write=None, *a, **k):
+        if fp_write is not None:
+            fp_write("\nTqdmDeprecationWarning: " + str(msg).rstrip() + '\n')
+        else:
+            super(TqdmDeprecationWarning, self).__init__(msg, *a, **k)
+
+
+class TMonitor(Thread):
+    """
+    Monitoring thread for tqdm bars.
+    Monitors if tqdm bars are taking too much time to display
+    and readjusts miniters automatically if necessary.
+
+    Parameters
+    ----------
+    tqdm_cls  : class
+        tqdm class to use (can be core tqdm or a submodule).
+    sleep_interval  : fload
+        Time to sleep between monitoring checks.
+    """
+
+    # internal vars for unit testing
+    _time = None
+    _sleep = None
+
+    def __init__(self, tqdm_cls, sleep_interval):
+        # setcheckinterval is deprecated
+        getattr(sys, 'setswitchinterval',
+                getattr(sys, 'setcheckinterval'))(100)
+        Thread.__init__(self)
+        self.daemon = True  # kill thread when main killed (KeyboardInterrupt)
+        self.was_killed = False
+        self.woken = 0  # last time woken up, to sync with monitor
+        self.tqdm_cls = tqdm_cls
+        self.sleep_interval = sleep_interval
+        if TMonitor._time is not None:
+            self._time = TMonitor._time
+        else:
+            self._time = time
+        if TMonitor._sleep is not None:
+            self._sleep = TMonitor._sleep
+        else:
+            self._sleep = sleep
+        self.start()
+
+    def exit(self):
+        self.was_killed = True
+        # self.join()  # DO NOT, blocking event, slows down tqdm at closing
+        return self.report()
+
+    def run(self):
+        cur_t = self._time()
+        while True:
+            # After processing and before sleeping, notify that we woke
+            # Need to be done just before sleeping
+            self.woken = cur_t
+            # Sleep some time...
+            self._sleep(self.sleep_interval)
+            # Quit if killed
+            # if self.exit_event.is_set():  # TODO: should work but does not...
+            if self.was_killed:
+                return
+            # Then monitor!
+            cur_t = self._time()
+            # Check for each tqdm instance if one is waiting too long to print
+            # NB: copy avoids size change during iteration RuntimeError
+            for instance in self.tqdm_cls._instances.copy():
+                # Only if mininterval > 1 (else iterations are just slow)
+                # and last refresh was longer than maxinterval in this instance
+                if instance.miniters > 1 and \
+                  (cur_t - instance.last_print_t) >= instance.maxinterval:
+                    # We force bypassing miniters on next iteration
+                    # dynamic_miniters should adjust mininterval automatically
+                    instance.miniters = 1
+                    # Refresh now! (works only for manual tqdm)
+                    instance.refresh()
+
+    def report(self):
+        # return self.is_alive()  # TODO: does not work...
+        return not self.was_killed
 
 
 class tqdm(object):
@@ -28,6 +125,10 @@ class tqdm(object):
     like the original iterable, but prints a dynamically updating
     progressbar every time a value is requested.
     """
+
+    monitor_interval = 10  # set to 0 to disable the thread
+    monitor = None
+
     @staticmethod
     def format_sizeof(num, suffix=''):
         """
@@ -85,25 +186,24 @@ class tqdm(object):
         updating may not work (it will print a new line at each refresh).
         """
         fp = file
-        if not getattr(fp, 'flush', False):  # pragma: no cover
-            fp.flush = lambda: None
+        fp_flush = getattr(fp, 'flush', lambda: None)  # pragma: no cover
 
         def fp_write(s):
             fp.write(_unicode(s))
+            fp_flush()
 
-        last_printed_len = [0]  # closure over mutable variable (fast)
+        last_len = [0]
 
         def print_status(s):
             len_s = len(s)
-            fp_write('\r' + s + (' ' * max(last_printed_len[0] - len_s, 0)))
-            fp.flush()
-            last_printed_len[0] = len_s
+            fp_write('\r' + s + (' ' * max(last_len[0] - len_s, 0)))
+            last_len[0] = len_s
         return print_status
 
     @staticmethod
     def format_meter(n, total, elapsed, ncols=None, prefix='',
                      ascii=False, unit='it', unit_scale=False, rate=None,
-                     bar_format=None):
+                     bar_format=None, postfix=None):
         """
         Return a string-based progress bar given some parameters
 
@@ -141,9 +241,11 @@ class tqdm(object):
             Specify a custom bar string formatting. May impact performance.
             [default: '{l_bar}{bar}{r_bar}'], where l_bar is
             '{desc}{percentage:3.0f}%|' and r_bar is
-            '| {n_fmt}/{total_fmt} [{elapsed_str}<{remaining_str}, {rate_fmt}]'.
+            '| {n_fmt}/{total_fmt} [{elapsed}<{remaining}, {rate_fmt}]'
             Possible vars: bar, n, n_fmt, total, total_fmt, percentage,
             rate, rate_fmt, elapsed, remaining, l_bar, r_bar, desc.
+        postfix  : str, optional
+            Same as prefix but will be placed at the end as additional stats.
 
         Returns
         -------
@@ -188,12 +290,49 @@ class tqdm(object):
             # format the stats displayed to the left and right sides of the bar
             l_bar = (prefix if prefix else '') + \
                 '{0:3.0f}%|'.format(percentage)
-            r_bar = '| {0}/{1} [{2}<{3}, {4}]'.format(
-                    n_fmt, total_fmt, elapsed_str, remaining_str, rate_fmt)
+            r_bar = '| {0}/{1} [{2}<{3}, {4}{5}]'.format(
+                    n_fmt, total_fmt, elapsed_str, remaining_str, rate_fmt,
+                    ', '+postfix if postfix else '')
 
             if ncols == 0:
                 return l_bar[:-1] + r_bar[1:]
 
+            if bar_format:
+                # Custom bar formatting
+                # Populate a dict with all available progress indicators
+                bar_args = {'n': n,
+                            'n_fmt': n_fmt,
+                            'total': total,
+                            'total_fmt': total_fmt,
+                            'percentage': percentage,
+                            'rate': rate if inv_rate is None else inv_rate,
+                            'rate_noinv': rate,
+                            'rate_noinv_fmt': ((format_sizeof(rate)
+                                               if unit_scale else
+                                               '{0:5.2f}'.format(rate))
+                                               if rate else '?') + unit + '/s',
+                            'rate_fmt': rate_fmt,
+                            'elapsed': elapsed_str,
+                            'remaining': remaining_str,
+                            'l_bar': l_bar,
+                            'r_bar': r_bar,
+                            'desc': prefix if prefix else '',
+                            'postfix': ', '+postfix if postfix else '',
+                            # 'bar': full_bar  # replaced by procedure below
+                            }
+
+                # Interpolate supplied bar format with the dict
+                if '{bar}' in bar_format:
+                    # Format left/right sides of the bar, and format the bar
+                    # later in the remaining space (avoid breaking display)
+                    l_bar_user, r_bar_user = bar_format.split('{bar}')
+                    l_bar = l_bar_user.format(**bar_args)
+                    r_bar = r_bar_user.format(**bar_args)
+                else:
+                    # Else no progress bar, we can just format and return
+                    return bar_format.format(**bar_args)
+
+            # Formatting progress bar
             # space available for bar's display
             N_BARS = max(1, ncols - len(l_bar) - len(r_bar)) if ncols \
                 else 10
@@ -222,39 +361,27 @@ class tqdm(object):
                 full_bar = bar + \
                     ' ' * max(N_BARS - bar_length, 0)
 
-            if bar_format is None:
-                # Default bar format = fast display
-                return l_bar + full_bar + r_bar
-            else:
-                # Custom bar formatting
-                # Populate a dict with all available progress indicators
-                bar_args = {'bar': full_bar,
-                            'n': n,
-                            'n_fmt': n_fmt,
-                            'total': total,
-                            'total_fmt': total_fmt,
-                            'percentage': percentage,
-                            'rate': rate,
-                            'rate_fmt': rate_fmt,
-                            'elapsed': elapsed_str,
-                            'remaining': remaining_str,
-                            'l_bar': l_bar,
-                            'r_bar': r_bar,
-                            'desc': prefix if prefix else ''
-                            }
-                # Interpolate supplied bar format with the dict
-                return bar_format.format(**bar_args)
+            # Piece together the bar parts
+            return l_bar + full_bar + r_bar
 
         # no total: no progressbar, ETA, just progress stats
         else:
-            return (prefix if prefix else '') + '{0}{1} [{2}, {3}]'.format(
-                n_fmt, unit, elapsed_str, rate_fmt)
+            return (prefix if prefix else '') + '{0}{1} [{2}, {3}{4}]'.format(
+                n_fmt, unit, elapsed_str, rate_fmt,
+                ', '+postfix if postfix else '')
 
     def __new__(cls, *args, **kwargs):
+        # Create a new instance
         instance = object.__new__(cls)
+        # Add to the list of instances
         if "_instances" not in cls.__dict__:
             cls._instances = WeakSet()
         cls._instances.add(instance)
+        # Create the monitoring thread
+        if cls.monitor_interval and (cls.monitor is None or
+                                     not cls.monitor.report()):
+            cls.monitor = TMonitor(cls, cls.monitor_interval)
+        # Return the instance
         return instance
 
     @classmethod
@@ -279,31 +406,168 @@ class tqdm(object):
             for inst in cls._instances:
                 if inst.pos > instance.pos:
                     inst.pos -= 1
+            # Kill monitor if no instances are left
+            if not cls._instances and cls.monitor:
+                cls.monitor.exit()
+                del cls.monitor
+                cls.monitor = None
         except KeyError:
             pass
 
+    @classmethod
+    def write(cls, s, file=sys.stdout, end="\n"):
+        """
+        Print a message via tqdm (without overlap with bars)
+        """
+        fp = file
+
+        # Clear all bars
+        inst_cleared = []
+        for inst in getattr(cls, '_instances', []):
+            # Clear instance if in the target output file
+            # or if write output + tqdm output are both either
+            # sys.stdout or sys.stderr (because both are mixed in terminal)
+            if inst.fp == fp or all(f in (sys.stdout, sys.stderr)
+                                    for f in (fp, inst.fp)):
+                inst.clear()
+                inst_cleared.append(inst)
+        # Write the message
+        fp.write(s)
+        fp.write(end)
+        # Force refresh display of bars we cleared
+        for inst in inst_cleared:
+            # Avoid race conditions by checking that the instance started
+            if hasattr(inst, 'start_t'):  # pragma: nocover
+                inst.refresh()
+        # TODO: make list of all instances incl. absolutely positioned ones?
+
+    @classmethod
+    def pandas(tclass, *targs, **tkwargs):
+        """
+        Registers the given `tqdm` class with
+            pandas.core.
+            ( frame.DataFrame
+            | series.Series
+            | groupby.DataFrameGroupBy
+            | groupby.SeriesGroupBy
+            ).progress_apply
+
+        A new instance will be create every time `progress_apply` is called,
+        and each instance will automatically close() upon completion.
+
+        Parameters
+        ----------
+        targs, tkwargs  : arguments for the tqdm instance
+
+        Examples
+        --------
+        >>> import pandas as pd
+        >>> import numpy as np
+        >>> from tqdm import tqdm, tqdm_gui
+        >>>
+        >>> df = pd.DataFrame(np.random.randint(0, 100, (100000, 6)))
+        >>> tqdm.pandas(ncols=50)  # can use tqdm_gui, optional kwargs, etc
+        >>> # Now you can use `progress_apply` instead of `apply`
+        >>> df.groupby(0).progress_apply(lambda x: x**2)
+
+        References
+        ----------
+        https://stackoverflow.com/questions/18603270/
+        progress-indicator-during-pandas-operations-python
+        """
+        from pandas.core.frame import DataFrame
+        from pandas.core.series import Series
+        from pandas.core.groupby import DataFrameGroupBy
+        from pandas.core.groupby import SeriesGroupBy
+        from pandas.core.groupby import GroupBy
+        from pandas.core.groupby import PanelGroupBy
+        from pandas import Panel
+
+        deprecated_t = [tkwargs.pop('deprecated_t', None)]
+
+        def inner_generator(df_function='apply'):
+            def inner(df, func, *args, **kwargs):
+                """
+                Parameters
+                ----------
+                df  : (DataFrame|Series)[GroupBy]
+                    Data (may be grouped).
+                func  : function
+                    To be applied on the (grouped) data.
+                *args, *kwargs  : optional
+                    Transmitted to `df.apply()`.
+                """
+                # Precompute total iterations
+                total = getattr(df, 'ngroups', None)
+                if total is None:  # not grouped
+                    total = len(df) if isinstance(df, Series) \
+                        else df.size // len(df)
+                else:
+                    total += 1  # pandas calls update once too many
+
+                # Init bar
+                if deprecated_t[0] is not None:
+                    t = deprecated_t[0]
+                    deprecated_t[0] = None
+                else:
+                    t = tclass(*targs, total=total, **tkwargs)
+
+                # Define bar updating wrapper
+                def wrapper(*args, **kwargs):
+                    t.update()
+                    return func(*args, **kwargs)
+
+                # Apply the provided function (in *args and **kwargs)
+                # on the df using our wrapper (which provides bar updating)
+                result = getattr(df, df_function)(wrapper, *args, **kwargs)
+
+                # Close bar and return pandas calculation result
+                t.close()
+                return result
+            return inner
+
+        # Monkeypatch pandas to provide easy methods
+        # Enable custom tqdm progress in pandas!
+        Series.progress_apply = inner_generator()
+        SeriesGroupBy.progress_apply = inner_generator()
+        Series.progress_map = inner_generator('map')
+        SeriesGroupBy.progress_map = inner_generator('map')
+
+        DataFrame.progress_apply = inner_generator()
+        DataFrameGroupBy.progress_apply = inner_generator()
+        DataFrame.progress_applymap = inner_generator('applymap')
+
+        Panel.progress_apply = inner_generator()
+        PanelGroupBy.progress_apply = inner_generator()
+
+        GroupBy.progress_apply = inner_generator()
+        GroupBy.progress_aggregate = inner_generator('aggregate')
+        GroupBy.progress_transform = inner_generator('transform')
+
     def __init__(self, iterable=None, desc=None, total=None, leave=True,
                  file=sys.stderr, ncols=None, mininterval=0.1,
                  maxinterval=10.0, miniters=None, ascii=None, disable=False,
                  unit='it', unit_scale=False, dynamic_ncols=False,
                  smoothing=0.3, bar_format=None, initial=0, position=None,
+                 postfix=None,
                  gui=False, **kwargs):
         """
         Parameters
         ----------
         iterable  : iterable, optional
             Iterable to decorate with a progressbar.
-            Leave blank [default: None] to manually manage the updates.
+            Leave blank to manually manage the updates.
         desc  : str, optional
-            Prefix for the progressbar [default: None].
+            Prefix for the progressbar.
         total  : int, optional
-            The number of expected iterations. If not given, len(iterable)
-            is used if possible. As a last resort, only basic progress
-            statistics are displayed (no ETA, no progressbar). If `gui` is
-            True and this parameter needs subsequent updating, specify an
-            initial arbitrary large positive integer, e.g. int(9e9).
+            The number of expected iterations. If unspecified,
+            len(iterable) is used if possible. As a last resort, only basic
+            progress statistics are displayed (no ETA, no progressbar).
+            If `gui` is True and this parameter needs subsequent updating,
+            specify an initial arbitrary large positive integer,
+            e.g. int(9e9).
         leave  : bool, optional
-            If [default: True], removes all traces of the progressbar
+            If [default: True], keeps all traces of the progressbar
             upon termination of iteration.
         file  : `io.TextIOWrapper` or `io.StringIO`, optional
             Specifies where to output the progress messages
@@ -312,24 +576,33 @@ class tqdm(object):
         ncols  : int, optional
             The width of the entire output message. If specified,
             dynamically resizes the progressbar to stay within this bound.
-            If [default: None], attempts to use environment width. The
+            If unspecified, attempts to use environment width. The
             fallback is a meter width of 10 and no limit for the counter and
             statistics. If 0, will not print any meter (only stats).
         mininterval  : float, optional
-            Minimum progress update interval, in seconds [default: 0.1].
+            Minimum progress display update interval, in seconds [default: 0.1].
         maxinterval  : float, optional
-            Maximum progress update interval, in seconds [default: 10.0].
+            Maximum progress display update interval, in seconds [default: 10].
+            Automatically adjusts `miniters` to correspond to `mininterval`
+            after long display update lag. Only works if `dynamic_miniters`
+            or monitor thread is enabled.
         miniters  : int, optional
-            Minimum progress update interval, in iterations [default: None].
+            Minimum progress display update interval, in iterations.
+            If 0 and `dynamic_miniters`, will automatically adjust to equal
+            `mininterval` (more CPU efficient, good for tight loops).
+            If > 0, will skip display of specified number of iterations.
+            Tweak this and `mininterval` to get very efficient loops.
+            If your progress is erratic with both fast and slow iterations
+            (network, skipping items, etc) you should set miniters=1.
         ascii  : bool, optional
-            If [default: None] or false, use unicode (smooth blocks) to fill
+            If unspecified or False, use unicode (smooth blocks) to fill
             the meter. The fallback is to use ASCII characters `1-9 #`.
-        disable : bool
+        disable  : bool or None, optional
             Whether to disable the entire progressbar wrapper
-            [default: False].
+            [default: False]. If set to None, disable on non-TTY.
         unit  : str, optional
             String that will be used to define the unit of each iteration
-            [default: 'it'].
+            [default: it].
         unit_scale  : bool, optional
             If set, the number of iterations will be reduced/scaled
             automatically and a metric prefix following the
@@ -338,32 +611,39 @@ class tqdm(object):
         dynamic_ncols  : bool, optional
             If set, constantly alters `ncols` to the environment (allowing
             for window resizes) [default: False].
-        smoothing  : float
+        smoothing  : float, optional
             Exponential moving average smoothing factor for speed estimates
             (ignored in GUI mode). Ranges from 0 (average speed) to 1
             (current/instantaneous speed) [default: 0.3].
         bar_format  : str, optional
             Specify a custom bar string formatting. May impact performance.
-            [default: '{l_bar}{bar}{r_bar}'], where l_bar is
+            If unspecified, will use '{l_bar}{bar}{r_bar}', where l_bar is
             '{desc}{percentage:3.0f}%|' and r_bar is
-            '| {n_fmt}/{total_fmt} [{elapsed_str}<{remaining_str}, {rate_fmt}]'.
+            '| {n_fmt}/{total_fmt} [{elapsed}<{remaining}, {rate_fmt}]'
             Possible vars: bar, n, n_fmt, total, total_fmt, percentage,
             rate, rate_fmt, elapsed, remaining, l_bar, r_bar, desc.
-        initial : int, optional
+        initial  : int, optional
             The initial counter value. Useful when restarting a progress
             bar [default: 0].
         position  : int, optional
-            Specify the line offset to print this bar [default: 0].
+            Specify the line offset to print this bar (starting from 0)
+            Automatic if unspecified.
             Useful to manage multiple bars at once (eg, from threads).
+        postfix  : dict, optional
+            Specify additional stats to display at the end of the bar.
         gui  : bool, optional
             WARNING: internal parameter - do not use.
             Use tqdm_gui(...) instead. If set, will attempt to use
-            matplotlib animations for a graphical output [default: false].
+            matplotlib animations for a graphical output [default: False].
 
         Returns
         -------
         out  : decorated iterator.
         """
+
+        if disable is None and hasattr(file, "isatty") and not file.isatty():
+            disable = True
+
         if disable:
             self.iterable = iterable
             self.disable = disable
@@ -375,11 +655,11 @@ class tqdm(object):
             self.disable = True
             self.pos = self._get_free_pos(self)
             self._instances.remove(self)
-            raise (DeprecationWarning("nested is deprecated and"
-                                      " automated.\nUse position instead"
-                                      " for manual control")
-                   if "nested" in kwargs else
-                   Warning("Unknown argument(s): " + str(kwargs)))
+            raise (TqdmDeprecationWarning("""\
+`nested` is deprecated and automated. Use position instead for manual control.
+""", fp_write=getattr(file, 'write', sys.stderr.write))
+                if "nested" in kwargs else
+                TqdmKeyError("Unknown argument(s): " + str(kwargs)))
 
         # Preprocess the arguments
         if total is None and iterable is not None:
@@ -389,11 +669,11 @@ class tqdm(object):
                 total = None
 
         if ((ncols is None) and (file in (sys.stderr, sys.stdout))) or \
-                dynamic_ncols:
-            if dynamic_ncols:  # pragma: no cover
+                dynamic_ncols:  # pragma: no cover
+            if dynamic_ncols:
                 dynamic_ncols = _environ_cols_wrapper()
                 ncols = dynamic_ncols(file)
-            else:  # pragma: no cover
+            else:
                 ncols = _environ_cols_wrapper()(file)
 
         if miniters is None:
@@ -439,6 +719,9 @@ class tqdm(object):
         self.avg_time = None
         self._time = time
         self.bar_format = bar_format
+        self.postfix = None
+        if postfix:
+            self.set_postfix(**postfix)
 
         # Init the iterations counters
         self.last_print_n = initial
@@ -455,15 +738,20 @@ class tqdm(object):
                 self.moveto(self.pos)
             self.sp(self.format_meter(self.n, total, 0,
                     (dynamic_ncols(file) if dynamic_ncols else ncols),
-                    self.desc, ascii, unit, unit_scale, None, bar_format))
+                    self.desc, ascii, unit, unit_scale, None, bar_format,
+                    self.postfix))
             if self.pos:
                 self.moveto(-self.pos)
 
         # Init the time counter
-        self.start_t = self.last_print_t = self._time()
+        self.last_print_t = self._time()
+        # NB: Avoid race conditions by setting start_t at the very end of init
+        self.start_t = self.last_print_t
 
     def __len__(self):
-        return len(self.iterable) if self.iterable else self.total
+        return (self.iterable.shape[0] if hasattr(self.iterable, 'shape')
+                else len(self.iterable)) if self.iterable is not None \
+            else self.total
 
     def __enter__(self):
         return self
@@ -476,25 +764,21 @@ class tqdm(object):
         self.close()
 
     def __repr__(self):
-        return self.format_meter(self.n, self.total, time() - self.last_print_t,
+        return self.format_meter(self.n, self.total,
+                                 self._time() - self.start_t,
                                  self.ncols, self.desc, self.ascii, self.unit,
                                  self.unit_scale, 1 / self.avg_time
-                                 if self.avg_time else None, self.bar_format)
+                                 if self.avg_time else None, self.bar_format,
+                                 self.postfix)
 
     def __lt__(self, other):
-        # try:
         return self.pos < other.pos
-        # except AttributeError:
-        #     return self.start_t < other.start_t
 
     def __le__(self, other):
         return (self < other) or (self == other)
 
     def __eq__(self, other):
-        # try:
         return self.pos == other.pos
-        # except AttributeError:
-        #     return self.start_t == other.start_t
 
     def __ne__(self, other):
         return not (self == other)
@@ -542,23 +826,25 @@ class tqdm(object):
             try:
                 sp = self.sp
             except AttributeError:
-                raise DeprecationWarning('Please use tqdm_gui(...)'
-                                         ' instead of tqdm(..., gui=True)')
+                raise TqdmDeprecationWarning("""\
+Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
+""", fp_write=getattr(self.fp, 'write', sys.stderr.write))
 
             for obj in iterable:
                 yield obj
                 # Update and print the progressbar.
                 # Note: does not call self.update(1) for speed optimisation.
                 n += 1
-                delta_it = n - last_print_n
                 # check the counter first (avoid calls to time())
-                if delta_it >= miniters:
-                    cur_t = _time()
-                    delta_t = cur_t - last_print_t
+                if n - last_print_n >= self.miniters:
+                    miniters = self.miniters  # watch monitoring thread changes
+                    delta_t = _time() - last_print_t
                     if delta_t >= mininterval:
-                        elapsed = cur_t - start_t
+                        cur_t = _time()
+                        delta_it = n - last_print_n
+                        elapsed = cur_t - start_t  # optimised if in inner loop
                         # EMA (not just overall average)
-                        if smoothing:  # and delta_it
+                        if smoothing and delta_t and delta_it:
                             avg_time = delta_t / delta_it \
                                 if avg_time is None \
                                 else smoothing * delta_t / delta_it + \
@@ -573,34 +859,45 @@ class tqdm(object):
                             (dynamic_ncols(self.fp) if dynamic_ncols
                              else ncols),
                             self.desc, ascii, unit, unit_scale,
-                            1 / avg_time if avg_time else None, bar_format))
+                            1 / avg_time if avg_time else None, bar_format,
+                            self.postfix))
 
                         if self.pos:
                             self.moveto(-self.pos)
 
                         # If no `miniters` was specified, adjust automatically
-                        # to the maximum iteration rate seen so far.
+                        # to the max iteration rate seen so far between 2 prints
                         if dynamic_miniters:
-                            if maxinterval and delta_t > maxinterval:
-                                # Set miniters to correspond to maxinterval
-                                miniters = delta_it * maxinterval / delta_t
-                            elif mininterval and delta_t:
+                            if maxinterval and delta_t >= maxinterval:
+                                # Adjust miniters to time interval by rule of 3
+                                if mininterval:
+                                    # Set miniters to correspond to mininterval
+                                    miniters = delta_it * mininterval / delta_t
+                                else:
+                                    # Set miniters to correspond to maxinterval
+                                    miniters = delta_it * maxinterval / delta_t
+                            elif smoothing:
                                 # EMA-weight miniters to converge
                                 # towards the timeframe of mininterval
-                                miniters = smoothing * delta_it * mininterval \
-                                    / delta_t + (1 - smoothing) * miniters
+                                miniters = smoothing * delta_it * \
+                                              (mininterval / delta_t
+                                               if mininterval and delta_t
+                                               else 1) + \
+                                              (1 - smoothing) * miniters
                             else:
-                                miniters = smoothing * delta_it + \
-                                    (1 - smoothing) * miniters
+                                # Maximum nb of iterations between 2 prints
+                                miniters = max(miniters, delta_it)
 
                         # Store old values for next call
-                        last_print_n = n
-                        last_print_t = cur_t
+                        self.n = self.last_print_n = last_print_n = n
+                        self.last_print_t = last_print_t = cur_t
+                        self.miniters = miniters
 
             # Closing the progress bar.
             # Update some internal variables for close().
             self.last_print_n = last_print_n
             self.n = n
+            self.miniters = miniters
             self.close()
 
     def update(self, n=1):
@@ -627,26 +924,27 @@ class tqdm(object):
             return
 
         if n < 0:
-            raise ValueError("n ({0}) cannot be less than 1".format(n))
+            raise ValueError("n ({0}) cannot be negative".format(n))
         self.n += n
 
-        delta_it = self.n - self.last_print_n  # should be n?
-        if delta_it >= self.miniters:
+        if self.n - self.last_print_n >= self.miniters:
             # We check the counter first, to reduce the overhead of time()
-            cur_t = self._time()
-            delta_t = cur_t - self.last_print_t
+            delta_t = self._time() - self.last_print_t
             if delta_t >= self.mininterval:
+                cur_t = self._time()
+                delta_it = self.n - self.last_print_n  # should be n?
                 elapsed = cur_t - self.start_t
                 # EMA (not just overall average)
-                if self.smoothing:  # and delta_it
+                if self.smoothing and delta_t and delta_it:
                     self.avg_time = delta_t / delta_it \
                         if self.avg_time is None \
                         else self.smoothing * delta_t / delta_it + \
                         (1 - self.smoothing) * self.avg_time
 
                 if not hasattr(self, "sp"):
-                    raise DeprecationWarning('Please use tqdm_gui(...)'
-                                             ' instead of tqdm(..., gui=True)')
+                    raise TqdmDeprecationWarning("""\
+Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
+""", fp_write=getattr(self.fp, 'write', sys.stderr.write))
 
                 if self.pos:
                     self.moveto(self.pos)
@@ -658,27 +956,32 @@ class tqdm(object):
                      else self.ncols),
                     self.desc, self.ascii, self.unit, self.unit_scale,
                     1 / self.avg_time if self.avg_time else None,
-                    self.bar_format))
+                    self.bar_format, self.postfix))
 
                 if self.pos:
                     self.moveto(-self.pos)
 
                 # If no `miniters` was specified, adjust automatically to the
-                # maximum iteration rate seen so far.
+                # maximum iteration rate seen so far between two prints.
                 # e.g.: After running `tqdm.update(5)`, subsequent
                 # calls to `tqdm.update()` will only cause an update after
                 # at least 5 more iterations.
                 if self.dynamic_miniters:
-                    if self.maxinterval and delta_t > self.maxinterval:
-                        self.miniters = self.miniters * self.maxinterval \
-                            / delta_t
-                    elif self.mininterval and delta_t:
-                        self.miniters = self.smoothing * delta_it \
-                            * self.mininterval / delta_t + \
-                            (1 - self.smoothing) * self.miniters
+                    if self.maxinterval and delta_t >= self.maxinterval:
+                        if self.mininterval:
+                            self.miniters = delta_it * self.mininterval \
+                                        / delta_t
+                        else:
+                            self.miniters = delta_it * self.maxinterval \
+                                        / delta_t
+                    elif self.smoothing:
+                        self.miniters = self.smoothing * delta_it * \
+                                        (self.mininterval / delta_t
+                                         if self.mininterval and delta_t
+                                         else 1) + \
+                                        (1 - self.smoothing) * self.miniters
                     else:
-                        self.miniters = self.smoothing * delta_it + \
-                            (1 - self.smoothing) * self.miniters
+                        self.miniters = max(self.miniters, delta_it)
 
                 # Store old values for next call
                 self.last_print_n = self.n
@@ -725,7 +1028,7 @@ class tqdm(object):
                     (self.dynamic_ncols(self.fp) if self.dynamic_ncols
                      else self.ncols),
                     self.desc, self.ascii, self.unit, self.unit_scale, None,
-                    self.bar_format))
+                    self.bar_format, self.postfix))
             if pos:
                 self.moveto(-pos)
             else:
@@ -751,9 +1054,61 @@ class tqdm(object):
         """
         self.desc = desc + ': ' if desc else ''
 
+    def set_postfix(self, ordered_dict=None, **kwargs):
+        """
+        Set/modify postfix (additional stats)
+        with automatic formatting based on datatype.
+        """
+        # Sort in alphabetical order to be more deterministic
+        postfix = _OrderedDict([] if ordered_dict is None else ordered_dict)
+        for key in sorted(kwargs.keys()):
+            postfix[key] = kwargs[key]
+        # Preprocess stats according to datatype
+        for key in postfix.keys():
+            # Number: limit the length of the string
+            if isinstance(postfix[key], Number):
+                postfix[key] = '{0:2.3g}'.format(postfix[key])
+            # Else for any other type, try to get the string conversion
+            elif not isinstance(postfix[key], _basestring):
+                postfix[key] = str(postfix[key])
+            # Else if it's a string, don't need to preprocess anything
+        # Stitch together to get the final postfix
+        self.postfix = ', '.join(key + '=' + postfix[key].strip()
+                                 for key in postfix.keys())
+
     def moveto(self, n):
         self.fp.write(_unicode('\n' * n + _term_move_up() * -n))
 
+    def clear(self, nomove=False):
+        """
+        Clear current bar display
+        """
+        if self.disable:
+            return
+
+        if not nomove:
+            self.moveto(self.pos)
+        # clear up the bar (can't rely on sp(''))
+        self.fp.write('\r')
+        self.fp.write(' ' * (self.ncols if self.ncols else 10))
+        self.fp.write('\r')  # place cursor back at the beginning of line
+        if not nomove:
+            self.moveto(-self.pos)
+
+    def refresh(self):
+        """
+        Force refresh the display of this bar
+        """
+        if self.disable:
+            return
+
+        self.moveto(self.pos)
+        # clear up this line's content (whatever there was)
+        self.clear(nomove=True)
+        # Print current/last bar state
+        self.fp.write(self.__repr__())
+        self.moveto(-self.pos)
+
 
 def trange(*args, **kwargs):
     """
